@@ -17,7 +17,7 @@ try:
 
     cfg = get_ipython()
     in_notebook = cfg is not None
-except (AttributeError, ImportError):
+except (AttributeError, ImportError, ModuleNotFoundError):
     in_notebook = False
 
 clight = speed_of_light * 1e-3
@@ -91,10 +91,13 @@ class FinalPlot:
             "sliders": [{"active": self.segment, "steps": steps}],
             "legend": {"traceorder": "reversed"},
         }
-        self.fig = go.FigureWidget(data=data, layout=layout)
+        if in_notebook:
+            self.fig = go.FigureWidget(data=data, layout=layout)
 
-        # add selection callback
-        self.fig.data[0].on_selection(self.selection_fn)
+            # add selection callback
+            self.fig.data[0].on_selection(self.selection_fn)
+        else:
+            self.fig = go.Figure(data=data, layout=layout)
 
         # Add button to save figure
         if in_notebook:
@@ -162,7 +165,11 @@ class FinalPlot:
             # Annoying I know, but plotly doesn't seem to have good controls for the z order
             # Or Legend order for that matter
 
-            if self.mask is not None:
+            if (
+                self.mask is not None
+                and self.spec is not None
+                and self.wave is not None
+            ):
                 # Line mask
                 x, y = self.create_mask_points(
                     self.wave[seg], self.spec[seg], self.mask[seg], 1
@@ -232,7 +239,7 @@ class FinalPlot:
                 visible += [seg]
 
             # mark important lines
-            if self.lines is not None:
+            if self.lines is not None and len(self.lines) != 0:
                 seg_annotations = []
                 xlimits = self.wave[seg][[0, -1]]
                 xlimits *= 1 - self.vrad[seg] / clight
@@ -241,13 +248,41 @@ class FinalPlot:
                 )
                 lines = self.lines[lines]
 
-                # Keep only the 100 stongest lines for performance
-                lines.sort("depth", ascending=False)
-                lines = lines[
-                    :20,
-                ]
+                # Filter out closely packaged lines of the same species
+                # Threshold for the distance between lines
+                wlcent, labels = [], []
+                threshold = np.diff(xlimits)[0] / 100
+                species = np.unique(lines["species"])
+                for sp in species:
+                    sp_lines = lines["wlcent"][lines["species"] == sp]
+                    diff = np.diff(sp_lines)
+                    sp_mask = diff < threshold
+                    if np.any(sp_mask):
+                        idx = np.where(np.diff(sp_mask))[0]
+                        idx = [0, *idx, len(sp_mask) + 1]
+                        idx = np.unique(idx)
+                        for i, j in zip(idx[:-1], idx[1:]):
+                            sp_wmid = np.mean(sp_lines[i:j])
+                            sp_label = f"{sp} +{j-i-1}"
+                            wlcent += [sp_wmid]
+                            labels += [sp_label]
+                    else:
+                        for sp_wmid in sp_lines:
+                            wlcent += [sp_wmid]
+                            labels += [sp]
 
-                x = lines.wlcent * (1 + self.vrad[seg] / clight)
+                wlcent = np.array(wlcent)
+                labels = np.array(labels)
+
+                # Keep only the 100 stongest lines for performance
+                # if "depth" in lines.columns:
+                #     lines.sort("depth", ascending=False)
+                #     lines = lines[:20]
+                # else:
+                #     idx = np.random.choice(len(lines), 20, replace=False)
+                #     lines = lines[idx]
+
+                x = wlcent * (1 + self.vrad[seg] / clight)
                 if self.spec is not None:
                     y = np.interp(x, self.wave[seg], self.spec[seg])
                 else:
@@ -260,15 +295,15 @@ class FinalPlot:
                 else:
                     ytop = 1
 
-                for i, line in enumerate(lines):
+                for i, line in enumerate(labels):
                     seg_annotations += [
                         {
                             "x": x[i],
                             "y": y[i],
                             "xref": "x",
                             "yref": "y",
-                            "text": f"{line.species}",
-                            "hovertext": f"{line.wlcent}",
+                            "text": f"{labels[i]}",
+                            "hovertext": f"{wlcent[i]}",
                             "textangle": 90,
                             "opacity": 1,
                             "ax": 0,
